import { useEffect, useState } from "react"
import { FetchPost, FetchPost_t, MediaPathTrc } from "./api"
import { z } from 'zod/v4'

type fetched = z.infer<typeof FetchPost_t>

export const Fetch = () => {
	let [fetched, setFetched] = useState<fetched>([])
	useEffect(() => {
		FetchPost().then(e => {
			if (e) {
				setFetched(e)
			}
		}
	)
	}, [])
	return (<div className="min-h-dvh w-dvw flex flex-col items-center gap-6 py-8 px-4">
		{fetched.map(post => <div className="w-full max-w-2xl bg-gray-200 rounded-lg shadow-md p-6 flex flex-col gap-4">
			<p className="text-gray-800 text-base leading-relaxed">{post.content}</p>
			{post.enclosure.length > 0 && post.enclosure[0] != null && <div className="flex flex-row overflow-x-auto gap-3 py-2 -mx-2 px-2 border-t border-gray-200 pt-4">
				{post.enclosure.map((enc, index) => {
					if (enc) {
						return <img
									key={index} 
									src={MediaPathTrc(enc)}
									alt={`Post media ${post.id}-${index}`}
									className="rounded-md h-32 object-cover aspect-video flex-shrink-0" 
								/>
					} else {
						return ""
					}
				})}
			</div>}
			{post.signing && <p className="text-right text-sm text-gray-600 italic mt-2">by: {post.signing}</p>}
			<div className="flex justify-between items-center text-xs text-gray-500 border-t border-gray-100 pt-3">
				<p className="self-start">PID: {post.id}</p>
				<p className="self-end">發文於: {post.post_at.toLocaleTimeString()}</p>
			</div>
		</div>)}
	</div>)
}
