import { useState, type ReactElement } from 'react'
import { useForm, type SubmitHandler } from 'react-hook-form'
import ImageWithPreview from './ImageWithPrview'
import { Post } from './api'
import { ShowCred } from './ShowCred'

type Inputs = {
  content: string
  signing?: string
  files?: FileList
}

export default function PostForm() {
  const {
    register,
    handleSubmit,
    watch,
    setValue,
    reset,
    formState: { errors },
  } = useForm<Inputs>()
  const onSubmit: SubmitHandler<Inputs> = (data) => {
    setPosting(true)
    console.log(data)
    Post(data).then((e) => setCred(e)).finally(() => setPosting(false))
  }

  const files_list = watch('files', undefined)
  const iter_file_list = (
    li: FileList,
    callback: (f: File, id: number) => any
  ) => {
    let agg = []
    let id = 0
    for (const f of li) {
      agg.push(callback(f, id))
      id += 1
    }
    return agg
  }
  const img_list: Array<ReactElement> = files_list
    ? iter_file_list(files_list, (f, id) => {
      return (
        <ImageWithPreview
          key={id}
          src={URL.createObjectURL(f)}
          remove={() => {
            const dt = new DataTransfer()
            for (const file of files_list) {
              if (f != file) {
                dt.items.add(file)
              }
            }
            setValue('files', dt.files)
          }}
        />
      )
    })
    : []
  const [cred, setCred] = useState<{ id: number; hash: string } | undefined>(
    undefined
  )
  const [posting, setPosting] = useState(false)

  return (
    <>
      {cred !== undefined && (
        <ShowCred
          id={cred.id}
          hash={cred.hash}
          close={() => {
            setCred(undefined)
            reset()
          }}
        />
      )}
      <form
        onSubmit={handleSubmit(onSubmit)}
        className={
          'flex flex-col m-auto w-10/12 h-[80dvh] px-6 py-8 gap-6 rounded-lg shadow-xl bg-white ' + // Added background, padding, shadow, rounded corners
          (cred !== undefined ? ' hidden' : '')
        }
      >
        <h1 className="text-4xl font-bold text-gray-800 mb-4 text-center">發佈匿名</h1> {/* Larger, bolder title, centered */}
        <textarea
          className="p-4 border border-gray-300 rounded-lg h-5/12 w-full resize-y focus:outline-none focus:border-gray-500 transition-colors duration-200 text-black" // Modernized textarea
          placeholder="匿名文章"
          {...register('content', { required: true })}
        />
        <div>
          <label htmlFor="files" className="block text-lg font-medium text-gray-700 mb-2 cursor-pointer"> {/* Styled label */}
            圖片
            <span className="ml-2 px-3 py-1 bg-gray-100 text-gray-700 rounded-full text-sm hover:bg-gray-200 transition-colors duration-200">選擇圖片</span>
          </label>
          <input
            multiple
            id="files"
            type="file"
            accept="image/png, image/jpeg, image/gif, image/webp, image/heif"
            className="hidden"
            {...register('files')}
          />
          <div
            className={
              'h-24 w-full gap-3 overflow-x-auto flex flex-row items-center border border-gray-200 rounded-lg p-2 ' + // Adjusted image preview container
              (files_list && files_list.length > 0 ? '' : 'hidden')
            }
          >
            {...img_list}
          </div>
        </div>

        <div className="ml-auto flex items-center gap-3"> {/* Aligned items, added items-center */}
          <label className="w-fit text-lg font-medium text-gray-700 break-keep">署名</label>
          <input
            className="border-b-2 border-gray-400 w-full text-right outline-none focus:border-gray-600 transition-colors duration-200 p-1" // Modernized signing input
            {...register('signing', {
              maxLength: 20,
            })}
          />
        </div>
        {errors.content && <span className="text-red-500 text-sm">此欄位為必填</span>} {/* Styled error message */}

        <button
          type="submit"
          className="mt-6 w-full py-3 bg-gray-800 text-white font-semibold rounded-lg hover:bg-gray-700 transition-colors duration-200 disabled:opacity-50 disabled:cursor-not-allowed" // Modernized button
          disabled={posting}
        >
          發送
        </button>
      </form>
    </>
  )
}